import os
import json
import shutil
from crontab import CronTab


DEFAULT_CONF_FILE = os.path.join(os.path.dirname(__file__), 'CONFIG.JSON')
CONF_FILE = os.path.join(os.path.expanduser('~'), 'CONFIG.JSON')

def get_serial_number():
    # dummy versions
    versions = dict(
        hardware='xxxxxxx',
        revision='0000000',
        serial='0000000000000000'
    )

    # open cpu info and read
    try:
        with open('/proc/cpuinfo', 'r') as info:
            for line in info:
                if line.startswith('Hardware'):
                    versions['hardware'] = line.split(':')[1].strip()
                elif line.startswith('Revision'):
                    versions['revision'] = line.split(':')[1].strip()
                elif line.startswith('Serial'):
                    versions['serial'] = line.split(':')[1].strip()

    except:
        versions = dict(
        hardware='ERRORxx',
        revision='ERROR00',
        serial='ERROR00000000000'
    )

    return versions


def parse_interval_to_seconds(s: str) -> int:
    s = s.lower()

    # Hours
    if 'hrs' in s:
        s = s.replace('hrs', 'h')
    if 'h' in s:
        s = s.replace('h', '')
        t = int(s)
        return t * 3600
    
    # Minutes
    if 'min' in s:
        s = s.replace('min', 'm')
    if 'm' in s:
        s = s.replace('m', '')
        t = int(s)
        return t * 60

    # Seconds
    if 'sec' in s:
        s = s.replace('sec', 's')
    if 's' in s:
        s = s.replace('s', '')
        return int(s)


def config(**kwargs) -> dict:
    if not os.path.exists(CONF_FILE):
        shutil.copy(DEFAULT_CONF_FILE, CONF_FILE)
    
    # get the config
    with open(CONF_FILE, 'r') as f:
        conf = json.load(f)

    # check if we are in read or write mode
    if len(kwargs.keys()) == 0:
        return conf 
    else:
        conf.update(kwargs)

        # write
        with open(CONF_FILE, 'w') as f:
            json.dump(conf, f, indent=4)
        
        return conf


def reset_config():
    shutil.copy(DEFAULT_CONF_FILE, CONF_FILE)


def enable_w1():
    try:
        if os.geteuid() != 0:
            raise AttributeError
    except AttributeError:
        print('You need root privileges on a UNIX OS to run this command.\nRun again like:\nsudo python3 -m raspi_logger enable_w1')
        return

    # get the script location
    PATH = os.path.abspath(os.path.join(os.path.dirname(__file__), 'enable_w1.sh'))
    
    # we run with sudo
    cron = CronTab(user='root')
    job = cron.new(command='%s' % PATH)
    job.every_reboot()
    cron.write()

    print('OneWire enabled.')



from crontab import CronTab

from .util import config, parse_interval_to_seconds, reset_config
from .logger import save_data


# maybe activate a a sensor protocol directly? for multiple cronjobs?
def activate(basecmd='python3 -m raspi_logger run'):
    cmt = 'raspi-logger-cron'
    # set the interval
    conf = config()
    interval = parse_interval_to_seconds(conf.get('loggerInterval', '15min'))
    if interval < 60:
        # less than a minute is not supported by crontab
        seconds = list(range(0, 60, interval))

        # get the crontab
        cron = CronTab(user=True)
        cmds = ['sleep %d;%s' % (s, basecmd) for s in seconds]
        jobs = [cron.new(command=cmd, comment=cmt) for cmd in cmds]
        for job in jobs:
            job.minute.every(1)
    else:
        job = cron.new(command=basecmd, comment=cmt)
        job.minute.every(int(interval / 60))
    
    # save 
    cron.write()

    # change config
    config(loggerCronjob='enabled')
    print('Saved.')


def deactivate(comment='raspi-logger-cron'):
    # disable the config first
    config(loggerCronjob='disabled')

    # get the crontab
    cron = CronTab(user=True)
    jobs = list(cron.find_comment(comment))

    # delete all
    for job in jobs:
        job.delete()
    
    cron.write()
    print('Stopped.')


def run():
    # first check if we should still log:
    conf = config()
    if conf.get('loggerCronjob', 'disabled')=='disabled':
        deactivate()
        return 

    # do the logging - TODO: here we could add a logic to handle more sensors
    save_data()


def settings(interval=None, enable=None, disable=None, enable_backend=None, disable_backend=None, reset=False):
    """
    Raspi Logger Settings\n
    Change the settings of the logger firmware, while the logger is running. 
    There is no need to shut the logger down. 
    Use this API for an IoT interface.
    :param interval: string - set a new measuring interval. E.g: '15min' or '4hrs'
        Using less than a Minute is experimental. Using less than 10sec can 
        cause the logger to crash with JSON backend.
    :param enable: If set, the logger will be activated. See activate command
    :param disable: IF set, the logger will be deactivated. See deactivate command
    :param enable_backend: string - Set the storage backend name to be enabed.
        Currenty a JSON file ('json') or a local sqlite DB ('sqlite') are supported.
    :param disable_backend: string - see enable_backend
    :param reset: If set, the config file will be completely reset.
        If you pass the reset flag along with other flags, these changes will be
        applied **after** the reset. 
    """
    if reset:
        reset_config()

    # check the settings
    # set new Interval
    if interval is not None:
        # TODO: validate the settings before writing
        config(loggerInterval=interval)

    # activate or deactivate the logger
    if isinstance(enable, bool) and enable:
        activate()
    if isinstance(disable, bool) and disable:
        deactivate()

    # check if a storage backend should be enabled
    backends = config().get('loggerBackends')
    if enable_backend is not None:
        backends[enable_backend]['enabled'] = True
    if disable_backend is not None:
        backends[disable_backend]['enabled'] = False
    config(loggerBackends=backends)

    return config()
    


import os

from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker, Session
from sqlalchemy import Column, BigInteger, String, Boolean


Base = declarative_base()


class RawData(Base):
    __tablename__ = 'raw_data'

    id = Column(BigInteger, primary_key=True)
    raw_dump = Column(String, nullable=False)
    uploaded = Column(Boolean, default=False)


def connect(path) -> Session:
    engine = create_engine(path)
    SessionCls = sessionmaker(bind=engine)

    return SessionCls()


def append_data(data, conf, path=None):
    # create the correct connection path
    if path is None:
        p = os.path.join(conf.get('loggerPath'), 'rawDataLogger.db')
        path = 'sqlite://{%s}' % p
    
    # if the db file does not exist, a call of create_all is needed
    if not os.path.exists(conf.get('loggerPath')):
        session = connect(path)
        Base.metadata.create_all(session.bind)
    else:
        # file was initialized before
        session = connect(path)

    # append the data
    if not isinstance(data, list):
        data = [data]
    
    try:
        session.add_all([RawData(raw_dump=record) for record in data])
        session.commit()
    except Exception as e:
        session.rollback()
        
        # TODO, some kind of logs would be necessary here
        print(str(e))
    
    return data
    



